@@ -1,13 +1,16 @@
-require "digest/sha1"
-require "log4r"
-require "pathname"
-require "uri"
+# Copyright (c) HashiCorp, Inc.
+# SPDX-License-Identifier: BUSL-1.1
 
-require "vagrant/box_metadata"
-require "vagrant/util/downloader"
-require "vagrant/util/file_checksum"
-require "vagrant/util/file_mutex"
-require "vagrant/util/platform"
+Vagrant.require "digest/sha1"
+Vagrant.require "log4r"
+Vagrant.require "pathname"
+Vagrant.require "uri"
+
+Vagrant.require "vagrant/box_metadata"
+Vagrant.require "vagrant/util/downloader"
+Vagrant.require "vagrant/util/file_checksum"
+Vagrant.require "vagrant/util/file_mutex"
+Vagrant.require "vagrant/util/platform"
 
 module Vagrant
   module Action
@@ -65,17 +68,29 @@ module Vagrant
           # If we received a shorthand URL ("mitchellh/precise64"),
           # then expand it properly.
           expanded = false
-          url.each_index do |i|
-            next if url[i] !~ /^[^\/]+\/[^\/]+$/
+          # Mark if only a single url entry was provided
+          single_entry = url.size == 1
 
-            if !File.file?(url[i])
+          url = url.map do |url_entry|
+            if url_entry =~ /^[^\/]+\/[^\/]+$/ && !File.file?(url_entry)
               server = Vagrant.server_url env[:box_server_url]
               raise Errors::BoxServerNotSet if !server
 
               expanded = true
-              url[i] = "#{server}/#{url[i]}"
+              # If only a single entry, expand to both the API endpoint and
+              # the direct shorthand endpoint.
+              if single_entry
+                url_entry = [
+                  "#{server}/api/v2/vagrant/#{url_entry}",
+                  "#{server}/#{url_entry}"
+                ]
+              else
+                url_entry = "#{server}/#{url_entry}"
+              end
             end
-          end
+
+            url_entry
+          end.flatten
 
           # Call the hook to transform URLs into authenticated URLs.
           # In the case we don't have a plugin that does this, then it
@@ -96,6 +111,21 @@ module Vagrant
             end
           end
 
+          # If only a single entry was provided, and it was expanded,
+          # inspect the metadata check results and extract the one that
+          # was successful, with preference to the API endpoint
+          if single_entry && expanded
+            idx = is_metadata_results.index { |v| v === true }
+            # If none of the urls were successful, set the index
+            # as the last entry
+            idx = is_metadata_results.size - 1 if idx.nil?
+
+            # Now reset collections with single value
+            is_metadata_results = [is_metadata_results[idx]]
+            authed_urls = [authed_urls[idx]]
+            url = [url[idx]]
+          end
+
           if expanded && url.length == 1
             is_error = is_metadata_results.find do |b|
               b.is_a?(Errors::DownloaderError)
@@ -162,6 +192,7 @@ module Vagrant
             env,
             checksum: env[:box_checksum],
             checksum_type: env[:box_checksum_type],
+            architecture: env[:architecture]
           )
         end
 
@@ -176,6 +207,9 @@ module Vagrant
         #   a Atlas server URL.
         def add_from_metadata(url, env, expanded)
           original_url = env[:box_url]
+          architecture = env[:box_architecture]
+          display_architecture = architecture == :auto ?
+                                   Util::Platform.architecture : architecture
           provider = env[:box_provider]
           provider = Array(provider) if provider
           version = env[:box_version]
@@ -225,12 +259,17 @@ module Vagrant
           end
 
           metadata_version  = metadata.version(
-            version || ">= 0", provider: provider)
+            version || ">= 0",
+            provider: provider,
+            architecture: architecture,
+          )
           if !metadata_version
-            if provider && !metadata.version(">= 0", provider: provider)
+            if provider && !metadata.version(">= 0", provider: provider, architecture: architecture)
               raise Errors::BoxAddNoMatchingProvider,
                 name: metadata.name,
-                requested: provider,
+                requested: [provider,
+                  display_architecture ? "(#{display_architecture})" : nil
+                ].compact.join(" "),
                 url: display_url
             else
               raise Errors::BoxAddNoMatchingVersion,
@@ -246,16 +285,16 @@ module Vagrant
             # If a provider was specified, make sure we get that specific
             # version.
             provider.each do |p|
-              metadata_provider = metadata_version.provider(p)
+              metadata_provider = metadata_version.provider(p, architecture)
               break if metadata_provider
             end
-          elsif metadata_version.providers.length == 1
+          elsif metadata_version.providers(architecture).length == 1
             # If we have only one provider in the metadata, just use that
             # provider.
             metadata_provider = metadata_version.provider(
-              metadata_version.providers.first)
+              metadata_version.providers.first, architecture)
           else
-            providers = metadata_version.providers.sort
+            providers = metadata_version.providers(architecture).sort
 
             choice = 0
             options = providers.map do |p|
@@ -276,7 +315,7 @@ module Vagrant
             end
 
             metadata_provider = metadata_version.provider(
-              providers[choice-1])
+              providers[choice-1], architecture)
           end
 
           provider_url = metadata_provider.url
@@ -290,6 +329,17 @@ module Vagrant
             provider_url = authed_urls[0]
           end
 
+          # The architecture name used when adding the box should be
+          # the value extracted from the metadata provider
+          arch_name = metadata_provider.architecture
+
+          # In the special case where the architecture name is "unknown" and
+          # it is listed as the default architecture, unset the architecture
+          # name so it is installed without architecture information
+          if arch_name == "unknown" && metadata_provider.default_architecture
+            arch_name = nil
+          end
+
           box_add(
             [[provider_url, metadata_provider.url]],
             metadata.name,
@@ -299,6 +349,7 @@ module Vagrant
             env,
             checksum: metadata_provider.checksum,
             checksum_type: metadata_provider.checksum_type,
+            architecture: arch_name,
           )
         end
 
@@ -314,16 +365,21 @@ module Vagrant
         # @param [Hash] env
         # @return [Box]
         def box_add(urls, name, version, provider, md_url, env, **opts)
+          display_architecture = opts[:architecture] == :auto ?
+                                   Util::Platform.architecture : opts[:architecture]
           env[:ui].output(I18n.t(
             "vagrant.box_add_with_version",
             name: name,
             version: version,
-            providers: Array(provider).join(", ")))
+            providers: [
+              provider,
+              display_architecture ? "(#{display_architecture})" : nil
+            ].compact.join(" ")))
 
           # Verify the box we're adding doesn't already exist
           if provider && !env[:box_force]
             box = env[:box_collection].find(
-              name, provider, version)
+              name, provider, version, opts[:architecture])
             if box
               raise Errors::BoxAlreadyExists,
                 name: name,
@@ -374,7 +430,9 @@ module Vagrant
               box_url, name, version,
               force: env[:box_force],
               metadata_url: md_url,
-              providers: provider)
+              providers: provider,
+              architecture: opts[:architecture]
+            )
           ensure
             # Make sure we delete the temporary file after we add it,
             # unless we were interrupted, in which case we keep it around
@@ -393,7 +451,10 @@ module Vagrant
             "vagrant.box_added",
             name: box.name,
             version: box.version,
-            provider: box.provider))
+            provider: [
+              provider,
+              display_architecture ? "(#{display_architecture})" : nil
+            ].compact.join(" ")))
 
           # Store the added box in the env for future middleware
           env[:box_added] = box
@@ -443,6 +504,7 @@ module Vagrant
           downloader_options[:headers] = ["Accept: application/json"] if opts[:json]
           downloader_options[:ui] = env[:ui] if opts[:ui]
           downloader_options[:location_trusted] = env[:box_download_location_trusted]
+          downloader_options[:disable_ssl_revoke_best_effort] = env[:box_download_disable_ssl_revoke_best_effort]
           downloader_options[:box_extra_download_options] = env[:box_extra_download_options]
 
           d = Util::Downloader.new(url, temp_path, downloader_options)
