@@ -1,9 +1,7 @@
 # Copyright (c) HashiCorp, Inc.
 # SPDX-License-Identifier: BUSL-1.1
 
-require "pathname"
-
-require "vagrant"
+Vagrant.require "pathname"
 
 module VagrantPlugins
   module HostRedHat
