@@ -1,9 +1,8 @@
 # Copyright (c) HashiCorp, Inc.
 # SPDX-License-Identifier: BUSL-1.1
 
-require "fileutils"
-
-require "log4r"
+Vagrant.require "fileutils"
+Vagrant.require "log4r"
 
 module VagrantPlugins
   module HyperV
