@@ -1,14 +1,14 @@
 # Copyright (c) HashiCorp, Inc.
 # SPDX-License-Identifier: BUSL-1.1
 
-require "ipaddr"
-require "resolv"
-require "set"
+Vagrant.require "ipaddr"
+Vagrant.require "resolv"
+Vagrant.require "set"
 
-require "log4r"
+Vagrant.require "log4r"
 
-require "vagrant/util/network_ip"
-require "vagrant/util/scoped_hash_override"
+Vagrant.require "vagrant/util/network_ip"
+Vagrant.require "vagrant/util/scoped_hash_override"
 
 module VagrantPlugins
   module ProviderVirtualBox
